module dsfml.system.alloc;

version (Tango)
{
    public import tango.core.Memory;
}
else
{
    import std.c.stdlib;
    import std.gc;
    
    struct GC
    {
        static void* malloc(uint size)
        {
            return std.c.stdlib.malloc(size);
        }
    
        static void free(void* ptr)
        {
            std.c.stdlib.free(ptr);
        }
    
        static void addRange(void* ptr, uint size)
        {
            std.gc.addRange(ptr, ptr + size);
        }
    
        static void removeRange(void* ptr)
        {
            std.gc.removeRange(ptr);
        }
    }
}

/*
*   Template for native non-GCed allocation for interaction between C and D threads.
*/
template Alloc()
{
    new (uint size)
    {
        void* p = GC.malloc(size);
        if (!p)
            assert(0);
        
        GC.addRange(p, size);
        return p;
    }

    delete(void* p)
    {
        GC.removeRange(p);
        GC.free(p);
    }
}
