/*
*   
*   
*   This file is part of DSFML
*
*   This software is provided 'as-is', without any express or
*   implied warranty. In no event will the authors be held
*   liable for any damages arising from the use of this software.
*
*   Permission is granted to anyone to use this software for any purpose,
*   including commercial applications, and to alter it and redistribute
*   it freely, subject to the following restrictions:
*
*   1.  The origin of this software must not be misrepresented;
*       you must not claim that you wrote the original software.
*       If you use this software in a product, an acknowledgment
*       in the product documentation would be appreciated but
*       is not required.
*
*   2.  Altered source versions must be plainly marked as such,
*       and must not be misrepresented as being the original software.
*
*   3.  This notice may not be removed or altered from any
*       source distribution.
*/

module dsfml.system.mutex;

import dsfml.system.pointerchecker;
import dsfml.system.external;

/**
*   Provide portable implementation of Mutual Exclusion object.
*   
*   Uses CriticalSection on Windows and pthread mutexes on linux.
*/
final class Mutex
{
    /**
    *   Constructor
    */   
    this()
    {
        m_mutex = sfMutex_Create();
    }

    ~this()
    {
        sfMutex_Destroy(m_mutex);
    }

    /**
    *   Lock the mutex
    *   
    *   Can be called multiples times, but each lock must be unlocked with unlock()        
    */        
    void lock()
    {
        sfMutex_Lock(m_mutex);
    }
    
    /**
    *   Unlock the mutex
    */        
    void unlock()
    {
        sfMutex_Unlock(m_mutex);
    }
private:
    sfMutex* m_mutex;
    
    mixin TEnforcePointer!(m_mutex);
}
